@@ -402,21 +402,17 @@ GnomeVFSResult
 corba_gnome_vfs_async_get_file_info (GnomeVFSAsyncHandle **handle_return,
 				     GList *uris,
 				     GnomeVFSFileInfoOptions options,
-				     const char * const meta_keys[],
 				     GnomeVFSAsyncGetFileInfoCallback callback,
 				     gpointer callback_data);
 GnomeVFSResult
 corba_gnome_vfs_async_get_file_info (GnomeVFSAsyncHandle **handle_return,
 				     GList *uris,
 				     GnomeVFSFileInfoOptions options,
-				     const char * const meta_keys[],
 				     GnomeVFSAsyncGetFileInfoCallback callback,
 				     gpointer callback_data)
 {
 	GnomeVFSSlaveProcess *slave;
 	GNOME_VFS_Slave_URIList *uri_list;
-	GNOME_VFS_MetadataKeyList my_meta_keys;
-	int num_meta_keys;
 	GnomeVFSAsyncDirectoryOpInfo *op_info; /* piggyback */
 
 	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
@@ -426,34 +422,18 @@ corba_gnome_vfs_async_get_file_info (GnomeVFSAsyncHandle **handle_return,
 	if (slave == NULL)
 		return GNOME_VFS_ERROR_INTERNAL;
 
-	COUNT_ELEMENTS (num_meta_keys, meta_keys, NULL);
-
 	op_info = &slave->op_info.directory;
-	op_info->num_meta_keys = num_meta_keys;
-	if (num_meta_keys == 0) {
-		op_info->meta_keys = NULL;
-	} else {
-		int i;
-		op_info->meta_keys = g_new (gchar *, num_meta_keys);
-		for (i = 0; i < num_meta_keys; i++)
-			op_info->meta_keys[i] = g_strdup (meta_keys[i]);
-	}
 
 	slave->callback = callback;
 	slave->callback_data = callback_data;
 
 	uri_list = gnome_vfs_uri_list_to_corba_uri_list (uris);
 
-	my_meta_keys._length = my_meta_keys._maximum = num_meta_keys;
-	my_meta_keys._buffer = (char **) meta_keys;
-	CORBA_sequence_set_release (&my_meta_keys, FALSE);
-
 	slave->operation_in_progress = GNOME_VFS_ASYNC_OP_GET_FILE_INFO;
 
 	GNOME_VFS_Slave_Request_get_file_info (slave->request_objref,
 					       uri_list,
 					       options,
-					       &my_meta_keys,
 					       &slave->ev);
 
 	CORBA_free (uri_list);
@@ -472,7 +452,6 @@ GnomeVFSResult
 corba_gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 				      const gchar *uri,
 				      GnomeVFSFileInfoOptions options,
-				      const gchar * const meta_keys[],
 				      GnomeVFSDirectorySortRule sort_rules[],
 				      gboolean reverse_order,
 				      GnomeVFSDirectoryFilterType filter_type,
@@ -485,7 +464,6 @@ GnomeVFSResult
 corba_gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 				      const gchar *uri,
 				      GnomeVFSFileInfoOptions options,
-				      const gchar * const meta_keys[],
 				      GnomeVFSDirectorySortRule sort_rules[],
 				      gboolean reverse_order,
 				      GnomeVFSDirectoryFilterType filter_type,
@@ -497,10 +475,8 @@ corba_gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 {
 	GNOME_VFS_Slave_DirectoryFilter my_filter;
 	GNOME_VFS_Slave_DirectorySortRuleList my_sort_rules;
-	GNOME_VFS_MetadataKeyList my_meta_keys;
 	GnomeVFSSlaveProcess *slave;
-	guint num_meta_keys, num_sort_rules;
-	guint i;
+	guint num_sort_rules;
 	GnomeVFSAsyncDirectoryOpInfo *op_info;
 
 	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
@@ -511,7 +487,6 @@ corba_gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 	if (slave == NULL)
 		return GNOME_VFS_ERROR_INTERNAL;
 
-	COUNT_ELEMENTS (num_meta_keys, meta_keys, NULL);
 	COUNT_ELEMENTS (num_sort_rules, sort_rules,
 			GNOME_VFS_DIRECTORY_SORT_NONE);
 
@@ -523,21 +498,9 @@ corba_gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 	op_info = &slave->op_info.directory;
 
 	op_info->list = NULL;
-	op_info->num_meta_keys = num_meta_keys;
-	if (num_meta_keys == 0) {
-		op_info->meta_keys = NULL;
-	} else {
-		op_info->meta_keys = g_new (gchar *, num_meta_keys);
-		for (i = 0; i < num_meta_keys; i++)
-			op_info->meta_keys[i] = g_strdup (meta_keys[i]);
-	}
 
 	/* Prepare the CORBA parameters.  */
 
-	my_meta_keys._length = my_meta_keys._maximum = num_meta_keys;
-	my_meta_keys._buffer = (char **)meta_keys;
-	CORBA_sequence_set_release (&my_meta_keys, FALSE);
-
 	my_filter.type = filter_type;
 	my_filter.options = filter_options;
 	if (filter_pattern == NULL)
@@ -556,7 +519,6 @@ corba_gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 	GNOME_VFS_Slave_Request_load_directory (slave->request_objref,
 						uri,
 						options,
-						&my_meta_keys,
 						&my_filter,
 						&my_sort_rules,
 						reverse_order,
@@ -577,7 +539,6 @@ GnomeVFSResult
 corba_gnome_vfs_async_load_directory_uri (GnomeVFSAsyncHandle **handle_return,
 					  GnomeVFSURI *uri,
 					  GnomeVFSFileInfoOptions options,
-					  const gchar * const meta_keys[],
 					  GnomeVFSDirectorySortRule sort_rules[],
 					  gboolean reverse_order,
 					  GnomeVFSDirectoryFilterType filter_type,
@@ -590,7 +551,6 @@ GnomeVFSResult
 corba_gnome_vfs_async_load_directory_uri (GnomeVFSAsyncHandle **handle_return,
 					  GnomeVFSURI *uri,
 					  GnomeVFSFileInfoOptions options,
-					  const gchar * const meta_keys[],
 					  GnomeVFSDirectorySortRule sort_rules[],
 					  gboolean reverse_order,
 					  GnomeVFSDirectoryFilterType filter_type,
@@ -605,13 +565,13 @@ corba_gnome_vfs_async_load_directory_uri (GnomeVFSAsyncHandle **handle_return,
 
 	str_uri = gnome_vfs_uri_to_string(uri, 0);
 
-	retval = corba_gnome_vfs_async_load_directory(handle_return, str_uri, options, meta_keys,
-						sort_rules, reverse_order, filter_type,
-						filter_options,
-						filter_pattern,
-						items_per_notification,
-						callback,
-						callback_data);
+	retval = corba_gnome_vfs_async_load_directory(handle_return, str_uri, options,
+						      sort_rules, reverse_order, filter_type,
+						      filter_options,
+						      filter_pattern,
+						      items_per_notification,
+						      callback,
+						      callback_data);
 
 	g_free(str_uri);
 
