@@ -416,6 +416,29 @@ dispatch_get_file_info_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 	g_list_free (result_list);
 }
 
+static void
+dispatch_set_file_info_callback (GnomeVFSJob *job, GnomeVFSOp *op)
+{
+	GnomeVFSAsyncSetFileInfoCallback callback;
+	gboolean new_info_is_valid;
+
+	new_info_is_valid = 
+		op->specifics.set_file_info.notify.set_file_info_result == GNOME_VFS_OK &&
+		op->specifics.set_file_info.notify.get_file_info_result == GNOME_VFS_OK;
+
+	callback = (GnomeVFSAsyncSetFileInfoCallback) op->callback;
+
+	(* callback) ((GnomeVFSAsyncHandle *) job,
+		      op->specifics.set_file_info.notify.set_file_info_result,
+		      new_info_is_valid 
+		      	? &op->specifics.set_file_info.notify.info 
+		      	: NULL,
+		      op->callback_data);
+
+	/* Always clear new info, whether or not it's valid. */
+	gnome_vfs_file_info_clear (&op->specifics.set_file_info.notify.info);
+}
+
 static void
 dispatch_xfer_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 {
@@ -511,6 +534,7 @@ dispatch_job_callback (GIOChannel *source,
 		case GNOME_VFS_OP_LOAD_DIRECTORY:
 		case GNOME_VFS_OP_XFER:
 		case GNOME_VFS_OP_GET_FILE_INFO:
+		case GNOME_VFS_OP_SET_FILE_INFO:
 			break;
 		}
 	} else {
@@ -548,6 +572,9 @@ dispatch_job_callback (GIOChannel *source,
 		case GNOME_VFS_OP_GET_FILE_INFO:
 			dispatch_get_file_info_callback (job, op);
 			break;
+		case GNOME_VFS_OP_SET_FILE_INFO:
+			dispatch_set_file_info_callback (job, op);
+			break;
 		default:
 			g_warning (_("Unknown job ID %d"), op->type);
 		}
@@ -1149,7 +1176,6 @@ execute_write (GnomeVFSJob *job)
 	return job_oneway_notify_and_close (job);
 }
 
-
 static gboolean
 execute_load_directory_not_sorted (GnomeVFSJob *job,
 				   GnomeVFSDirectoryFilter *filter)
@@ -1168,7 +1194,6 @@ execute_load_directory_not_sorted (GnomeVFSJob *job,
 		(&handle,
 		 load_directory_op->request.uri,
 		 load_directory_op->request.options,
-		 (const char **)load_directory_op->request.meta_keys,
 		 filter);
 
 	if (result != GNOME_VFS_OK) {
@@ -1256,7 +1281,6 @@ execute_load_directory_sorted (GnomeVFSJob *job,
 		(&directory_list,
 		 load_directory_op->request.uri,
 		 load_directory_op->request.options,
-		 (const char **)load_directory_op->request.meta_keys,
 		 filter);
 
 	if (result != GNOME_VFS_OK) {
@@ -1329,7 +1353,6 @@ execute_get_file_info (GnomeVFSJob *job)
 			(result_item->uri,
 			 result_item->file_info,
 			 gijob->request.options,
-			 (const char **)gijob->request.meta_keys,
 			 job->current_op->context);
 
 		gijob->notify.result_list = g_list_prepend
@@ -1337,7 +1360,33 @@ execute_get_file_info (GnomeVFSJob *job)
 	}
 	gijob->notify.result_list = g_list_reverse (gijob->notify.result_list);
 
-	g_strfreev (gijob->request.meta_keys);
+	job_oneway_notify_and_close (job);
+	return FALSE;
+}
+
+static gboolean
+execute_set_file_info (GnomeVFSJob *job)
+{
+	GnomeVFSSetFileInfoOp *op;
+
+	op = &job->current_op->specifics.set_file_info;
+
+	op->notify.set_file_info_result = gnome_vfs_set_file_info_cancellable
+		(op->request.uri, &op->request.info, op->request.mask,
+		 job->current_op->context);
+
+	/* FIXME: potential leak if operation is cancelled before getting here. */
+	gnome_vfs_file_info_clear (&op->request.info);
+
+	/* Always get new file info, even if setter failed. Init here and clear
+	 * in dispatch_set_file_info.
+	 */
+	gnome_vfs_file_info_init (&op->notify.info);
+	op->notify.get_file_info_result = gnome_vfs_get_file_info_uri_cancellable
+		(op->request.uri,
+		 &op->notify.info,
+		 op->request.options,
+		 job->current_op->context);
 
 	job_oneway_notify_and_close (job);
 	return FALSE;
@@ -1372,7 +1421,6 @@ execute_load_directory (GnomeVFSJob *job)
 	g_free (load_directory_op->request.sort_rules);
 	g_free (load_directory_op->request.filter_pattern);
 
-	g_strfreev(load_directory_op->request.meta_keys);
 
 	return FALSE;
 }
@@ -1483,6 +1531,8 @@ gnome_vfs_job_execute (GnomeVFSJob *job)
 		return execute_xfer (job);
 	case GNOME_VFS_OP_GET_FILE_INFO:
 		return execute_get_file_info (job);
+	case GNOME_VFS_OP_SET_FILE_INFO:
+		return execute_set_file_info (job);
 	default:
 		g_warning (_("Unknown job ID %d"), job->current_op->type);
 		return FALSE;
