@@ -123,15 +123,28 @@ void          	 gnome_vfs_async_write		(GnomeVFSAsyncHandle *handle,
 void             gnome_vfs_async_get_file_info  (GnomeVFSAsyncHandle **handle_return,
 						 GList *uri_list, /* GnomeVFSURI* items */
 						 GnomeVFSFileInfoOptions options,
-						 const gchar * const meta_keys[],
 						 GnomeVFSAsyncGetFileInfoCallback callback,
 						 gpointer callback_data);
 
+/* Setting the file info sometimes changes more info than the
+ * caller specified; for example, if the name changes the MIME type might
+ * change, and if the owner changes the SUID & SGID bits might change. 
+ * Therefore the callback returns the new file info for the caller's
+ * convenience. The GnomeVFSFileInfoOptions passed here are those used 
+ * for the returned file info; they are not used when setting.
+ */
+void             gnome_vfs_async_set_file_info  (GnomeVFSAsyncHandle **handle_return,
+						 GnomeVFSURI *uri,
+						 GnomeVFSFileInfo *info,
+						 GnomeVFSSetFileInfoMask mask,
+						 GnomeVFSFileInfoOptions options,
+						 GnomeVFSAsyncSetFileInfoCallback callback,
+						 gpointer callback_data);
+
 void          	 gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 						 const gchar *text_uri,
 						 GnomeVFSFileInfoOptions
 						         options,
-						 const gchar * const meta_keys[],
 						 GnomeVFSDirectorySortRule
 						 	sort_rules[],
 						 gboolean reverse_order,
@@ -150,7 +163,6 @@ void          	 gnome_vfs_async_load_directory_uri
 						 GnomeVFSURI *uri,
 						 GnomeVFSFileInfoOptions
 						         options,
-						 const gchar *meta_keys[],
 						 GnomeVFSDirectorySortRule
 						 	sort_rules[],
 						 gboolean reverse_order,
