@@ -26,6 +26,13 @@
 
 #include "gnome-vfs.h"
 
+#define GNOME_VFS_FILE_INFO_SYMLINK(info)		\
+	((info)->flags & GNOME_VFS_FILE_FLAGS_SYMLINK)
+
+#define GNOME_VFS_FILE_INFO_SET_SYMLINK(info, value)			\
+	(value ? ((info)->flags |= GNOME_VFS_FILE_FLAGS_SYMLINK)	\
+	       : ((info)->flags &= ~GNOME_VFS_FILE_FLAGS_SYMLINK))
+
 #define GNOME_VFS_FILE_INFO_LOCAL(info)			\
 	((info)->flags & GNOME_VFS_FILE_FLAGS_LOCAL)
 
@@ -65,19 +72,6 @@ void		 gnome_vfs_file_info_init	(GnomeVFSFileInfo *info);
 void		 gnome_vfs_file_info_clear	(GnomeVFSFileInfo *info);
 void 		 gnome_vfs_file_info_unref   	(GnomeVFSFileInfo *info);
 void 		 gnome_vfs_file_info_ref     	(GnomeVFSFileInfo *info);
-gboolean 	 gnome_vfs_file_info_get_metadata
-						(GnomeVFSFileInfo *info,
-						 const gchar *key,
-						 gconstpointer *value,
-						 guint *value_size);
-gboolean 	 gnome_vfs_file_info_set_metadata
-						(GnomeVFSFileInfo *info,
-						 const gchar *key,
-						 gpointer value,
-						 guint value_size);
-gboolean	 gnome_vfs_file_info_unset_metadata
-						(GnomeVFSFileInfo *info,
-						 const gchar *key);
 const gchar	*gnome_vfs_file_info_get_mime_type
 						(GnomeVFSFileInfo *info);
 
