@@ -54,14 +54,13 @@ static GList *gnome_vfs_strsplit_to_list (const char *str, const char *delim, in
 static char *gnome_vfs_strjoin_from_list (const char *separator, GList *list);
 static void g_list_free_deep (GList *list);
 static GList *prune_ids_for_nonexistent_applications (GList *list);
+static GnomeVFSResult gnome_vfs_mime_edit_user_file (const char *mime_type, const char *key, const char *value);
 
 
-GnomeVFSMimeActionType
-gnome_vfs_mime_get_default_action_type (const char *mime_type)
+static GnomeVFSMimeActionType
+gnome_vfs_mime_get_default_action_type_without_fallback (const char *mime_type)
 {
 	const char *action_type_string;
-	char *supertype;
-	GnomeVFSMimeActionType action_type;
 
 	action_type_string = gnome_vfs_mime_get_value
 		(mime_type, "default_action_type");
@@ -71,29 +70,37 @@ gnome_vfs_mime_get_default_action_type (const char *mime_type)
 	} else if (action_type_string != NULL && strcasecmp (action_type_string, "component") == 0) {
 		return GNOME_VFS_MIME_ACTION_TYPE_COMPONENT;
 	} else {
-		/* Fall back to the supertype. */
-		supertype = mime_type_get_supertype (mime_type);
-
-		/* Check if already a supertype */
-		if (supertype != NULL && strcmp (supertype, mime_type) != 0) {
-			action_type = gnome_vfs_mime_get_default_action_type (supertype);
-		} else {
-			action_type = GNOME_VFS_MIME_ACTION_TYPE_NONE;
-		}
-
-		g_free (supertype);
-		return action_type;
+		return GNOME_VFS_MIME_ACTION_TYPE_NONE;
 	}
 }
 
+GnomeVFSMimeActionType
+gnome_vfs_mime_get_default_action_type (const char *mime_type)
+{
+	char *supertype;
+	GnomeVFSMimeActionType action_type;
+
+	action_type = gnome_vfs_mime_get_default_action_type_without_fallback (mime_type);
+
+	if (action_type == GNOME_VFS_MIME_ACTION_TYPE_NONE) {
+		/* Fall back to the supertype. */
+		supertype = mime_type_get_supertype (mime_type);
+
+		action_type = gnome_vfs_mime_get_default_action_type_without_fallback (supertype);
+		g_free (supertype);
+	}
+
+	return action_type;
+}
+
 GnomeVFSMimeAction *
-gnome_vfs_mime_get_default_action (const char *mime_type)
+gnome_vfs_mime_get_default_action_without_fallback (const char *mime_type)
 {
 	GnomeVFSMimeAction *action;
 
 	action = g_new0 (GnomeVFSMimeAction, 1);
 
-	action->action_type = gnome_vfs_mime_get_default_action_type (mime_type);
+	action->action_type = gnome_vfs_mime_get_default_action_type_without_fallback (mime_type);
 
 	switch (action->action_type) {
 	case GNOME_VFS_MIME_ACTION_TYPE_APPLICATION:
@@ -127,6 +134,23 @@ gnome_vfs_mime_get_default_action (const char *mime_type)
 	return action;
 }
 
+GnomeVFSMimeAction *
+gnome_vfs_mime_get_default_action (const char *mime_type)
+{
+	GnomeVFSMimeAction *action;
+	char *supertype;
+
+	action = gnome_vfs_mime_get_default_action_without_fallback (mime_type);
+	if (action == NULL) {
+		/* Fall back to the supertype. */
+		supertype = mime_type_get_supertype (mime_type);
+		action = gnome_vfs_mime_get_default_action_without_fallback (supertype);
+		g_free (supertype);
+	}
+
+	return action;
+}
+
 GnomeVFSMimeApplication *
 gnome_vfs_mime_get_default_application (const char *mime_type)
 {
@@ -155,6 +179,19 @@ gnome_vfs_mime_get_default_application (const char *mime_type)
 	return gnome_vfs_mime_application_new_from_id (default_application_id);
 }
 
+const char *
+gnome_vfs_mime_get_icon (const char *mime_type)
+{
+	return (gnome_vfs_mime_get_value (mime_type, "icon-filename"));
+}
+
+GnomeVFSResult
+gnome_vfs_mime_set_icon (const char *mime_type, const char *filename)
+{
+	return gnome_vfs_mime_edit_user_file
+		(mime_type, "icon-filename", filename);
+}
+
 
 #ifdef USING_OAF
 OAF_ServerInfo *
@@ -1317,22 +1354,25 @@ gnome_vfs_mime_remove_component_from_short_list (const char *mime_type,
 GnomeVFSResult
 gnome_vfs_mime_add_extension (const char *mime_type, const char *extension)
 {
+	GnomeVFSResult result;
 	GList *list, *element;
 	gchar *extensions, *old_extensions;
 
 	extensions = NULL;
 	old_extensions = NULL;
 
-	list = gnome_vfs_mime_get_extensions (mime_type);	
+	result = GNOME_VFS_OK;
+	
+	list = gnome_vfs_mime_get_extensions_list (mime_type);	
 	if (list == NULL) {
-		return GNOME_VFS_OK;
+		return result;
 	}
 
 	/* Check for duplicates */
 	for (element = list; element != NULL; element = element->next) {
 		if (strcmp (extension, (char *)element->data) == 0) {					
-			gnome_vfs_mime_extension_list_free (list);
-			return GNOME_VFS_OK;
+			gnome_vfs_mime_extensions_list_free (list);
+			return result;
 		}
 	}
 
@@ -1354,13 +1394,12 @@ gnome_vfs_mime_add_extension (const char *mime_type, const char *extension)
 
 		/* Add extensions to hash table and flush into the file. */
 		gnome_vfs_mime_set_registered_type_key (mime_type, "ext", extensions);
-		/* FIXME: No possibility of error? */
-		gnome_vfs_mime_commit_registered_types ();
+		result = gnome_vfs_mime_commit_registered_types ();
 	}
 	
-	gnome_vfs_mime_extension_list_free (list);
+	gnome_vfs_mime_extensions_list_free (list);
 
-	return GNOME_VFS_OK;
+	return result;
 }
 
 GnomeVFSResult
@@ -1369,14 +1408,16 @@ gnome_vfs_mime_remove_extension (const char *mime_type, const char *extension)
 	GList *list, *element;
 	gchar *extensions, *old_extensions;
 	gboolean in_list;
-	
+	GnomeVFSResult result;
+
+	result = GNOME_VFS_OK;
 	extensions = NULL;
 	old_extensions = NULL;
 	in_list = FALSE;
 	
-	list = gnome_vfs_mime_get_extensions (mime_type);	
+	list = gnome_vfs_mime_get_extensions_list (mime_type);	
 	if (list == NULL) {
-		return GNOME_VFS_OK;
+		return result;
 	}
 
 	/* See if extension is in list */
@@ -1396,8 +1437,8 @@ gnome_vfs_mime_remove_extension (const char *mime_type, const char *extension)
 
 	/* Exit if we found no match */
 	if (!in_list) {
-		gnome_vfs_mime_extension_list_free (list);
-		return GNOME_VFS_OK;
+		gnome_vfs_mime_extensions_list_free (list);
+		return result;
 	}
 	
 	/* Create new extension list */
@@ -1414,13 +1455,12 @@ gnome_vfs_mime_remove_extension (const char *mime_type, const char *extension)
 	if (extensions != NULL) {
 		/* Add extensions to hash table and flush into the file */
 		gnome_vfs_mime_set_registered_type_key (mime_type, "ext", extensions);
-		/* FIXME: No possibility of error? */
-		gnome_vfs_mime_commit_registered_types ();
+		result = gnome_vfs_mime_commit_registered_types ();
 	}
 	
-	gnome_vfs_mime_extension_list_free (list);
+	gnome_vfs_mime_extensions_list_free (list);
 
-	return GNOME_VFS_OK;
+	return result;
 }
 
 
