@@ -61,6 +61,7 @@ typedef struct {
 GnomeVFSMimeApplication *gnome_vfs_mime_application_copy                   (GnomeVFSMimeApplication *application);
 GnomeVFSMimeActionType   gnome_vfs_mime_get_default_action_type            (const char              *mime_type);
 GnomeVFSMimeAction *     gnome_vfs_mime_get_default_action                 (const char              *mime_type);
+GnomeVFSMimeAction *     gnome_vfs_mime_get_default_action_without_fallback(const char              *mime_type);
 GnomeVFSMimeApplication *gnome_vfs_mime_get_default_application            (const char              *mime_type);
 #ifdef USING_OAF
 OAF_ServerInfo *         gnome_vfs_mime_get_default_component              (const char              *mime_type);
@@ -75,6 +76,9 @@ GnomeVFSResult           gnome_vfs_mime_set_default_application            (cons
 									    const char              *application_id);
 GnomeVFSResult           gnome_vfs_mime_set_default_component              (const char              *mime_type,
 									    const char              *component_iid);
+const char  		 *gnome_vfs_mime_get_icon 			   (const char 		    *mime_type);
+GnomeVFSResult  	 gnome_vfs_mime_set_icon 			   (const char 		    *mime_type,
+									    const char		    *filename);
 
 /* Stored as delta to current user level - API function computes delta and stores in prefs */
 GnomeVFSResult           gnome_vfs_mime_set_short_list_applications        (const char              *mime_type,
