@@ -25,6 +25,7 @@
 
 #include "gnome-vfs-mime.h"
 #include "gnome-vfs-mime-private.h"
+#include "gnome-vfs-result.h"
 
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
@@ -90,7 +91,7 @@ static GHashTable *generic_types;
 static GHashTable *registered_types;
 
 /* Prototypes */
-static void write_registered_mime_data (void);
+static GnomeVFSResult write_registered_mime_data (void);
 
 
 static GnomeMimeContext *
@@ -266,7 +267,7 @@ typedef enum {
 } ParserState;
 
 static void
-load_mime_type_info_from (char *filename)
+load_mime_type_info_from (const char *filename)
 {
 	FILE *mime_file;
 	gboolean in_comment, context_used;
@@ -438,7 +439,7 @@ load_mime_type_info_from (char *filename)
  */
  
 static void
-load_mime_list_info_from (char *filename)
+load_mime_list_info_from (const char *filename)
 {
 	FILE *mime_file;
 	gboolean in_comment, context_used;
@@ -734,7 +735,7 @@ load_mime_type_info (void)
 }
 
 static void
-gnome_vfs_mime_init ()
+gnome_vfs_mime_init (void)
 {
 	/*
 	 * The hash tables that store the mime keys.
@@ -745,17 +746,15 @@ gnome_vfs_mime_init ()
 	
 	current_lang = gnome_i18n_get_language_list ("LC_MESSAGES");
 	if(current_lang)
-		current_lang = g_list_reverse(current_lang);
+		current_lang = g_list_reverse(g_list_copy(current_lang));
 
 	/*
 	 * Setup the descriptors for the information loading
 	 */
 
-	/* FIXME bugzilla.eazel.com 796: Looks in gnome-libs prefix instead of gnome-vfs prefix. */
-	gnome_mime_dir.dirname = gnome_unconditional_datadir_file ("mime-info");
+	gnome_mime_dir.dirname = g_strconcat (GNOME_VFS_DATADIR, "/mime-info", NULL);
 	gnome_mime_dir.system_dir = TRUE;
 	
-	/* FIXME bugzilla.eazel.com 796: Looks in gnome-libs prefix instead of gnome-vfs prefix. */
 	user_mime_dir.dirname  = gnome_util_home_file ("mime-info");
 	user_mime_dir.system_dir = FALSE;
 
@@ -1002,13 +1001,13 @@ gnome_vfs_mime_description (const char *mime_type)
 }
 
 /**
- * gnome_vfs_mime_extensions:
+ * gnome_vfs_mime_extensions_list:
  * @mime_type: the mime type
  *
- * Returns the description for this mime-type
+ * Returns a list of extensions for this mime-type
  */
 GList *
-gnome_vfs_mime_get_extensions (const char *mime_type)
+gnome_vfs_mime_get_extensions_list (const char *mime_type)
 {
 	GList *list;
 	const char *extensions;
@@ -1051,6 +1050,94 @@ gnome_vfs_mime_get_extensions (const char *mime_type)
 	return list;
 }
 
+
+/**
+ * gnome_vfs_mime_extensions_string:
+ * @mime_type: the mime type
+ *
+ * Returns a string containing extensions for this mime-type
+ */
+char *
+gnome_vfs_mime_get_extensions_string (const char *mime_type)
+{
+	const char *extensions;
+	GnomeMimeContext *context;
+	
+	if (mime_type == NULL) {
+		return NULL;
+	}
+
+	if (!gnome_vfs_mime_inited) {
+		gnome_vfs_mime_init ();
+	}
+
+	maybe_reload ();
+	
+	context = g_hash_table_lookup (registered_types, mime_type);
+	if (context) {
+		extensions = g_hash_table_lookup (context->keys, "ext");		
+	}
+
+	if (extensions != NULL) {
+		return g_strdup (extensions);
+	}		
+
+	return NULL;
+}
+	
+/**
+ * gnome_vfs_mime_get_extensions_pretty_string:
+ * @mime_type: the mime type
+ *
+ * Returns a string containing comma seperated extensions for this mime-type
+ */
+char *
+gnome_vfs_mime_get_extensions_pretty_string (const char *mime_type)
+{
+	GList *extensions, *element;
+	char *ext_str, *tmp_str;
+	
+	if (mime_type == NULL) {
+		return NULL;
+	}
+
+	ext_str = NULL;
+	tmp_str = NULL;
+
+	if (!gnome_vfs_mime_inited) {
+		gnome_vfs_mime_init ();
+	}
+
+	maybe_reload ();
+	
+	extensions = gnome_vfs_mime_get_extensions_list (mime_type);
+	if (extensions != NULL) {
+		for (element = extensions; element != NULL; element = element->next) {
+			if (ext_str != NULL) {
+				tmp_str = ext_str;
+
+				if (element->next == NULL) {
+					ext_str = g_strconcat (tmp_str, ".", (char *)element->data, NULL);
+				} else {
+					ext_str = g_strconcat (tmp_str, ".", (char *)element->data, ", ", NULL);
+				}
+				g_free (tmp_str);
+			} else {
+				if (g_list_length (extensions) == 1) {
+					ext_str = g_strconcat (".", (char *)element->data, NULL);
+				} else {
+					ext_str = g_strconcat (".", (char *)element->data, ", ", NULL);
+				}
+			}
+		}
+
+		gnome_vfs_mime_extensions_list_free (extensions);
+	}
+
+	return ext_str;
+}
+
+
 /**
  * gnome_vfs_mime_extension_list_free:
  * @list: the extensions list
@@ -1059,7 +1146,7 @@ gnome_vfs_mime_get_extensions (const char *mime_type)
  * to free the list and all of its elements.
  */
 void
-gnome_vfs_mime_extension_list_free (GList *list)
+gnome_vfs_mime_extensions_list_free (GList *list)
 {
 	g_list_foreach (list, (GFunc) g_free, NULL);
 	g_list_free (list);
@@ -1197,10 +1284,10 @@ gnome_vfs_get_registered_mime_types (void)
  * hash table to disk.
  */
 
-void
+GnomeVFSResult
 gnome_vfs_mime_commit_registered_types (void)
 {
-	write_registered_mime_data ();
+	return write_registered_mime_data ();
 }
 
 /**
@@ -1255,7 +1342,7 @@ write_mime_data_foreach (gpointer key, gpointer value, gpointer data)
 	fwrite ("\n", 1, 1, file);
 }
 
-static void
+static GnomeVFSResult
 write_registered_mime_data (void)
 {
 	DIR *dir;
@@ -1272,13 +1359,13 @@ write_registered_mime_data (void)
 	dir = opendir (gnome_mime_dir.dirname);
 	if (!dir){
 		gnome_mime_dir.valid = FALSE;
-		return;
+		return gnome_vfs_result_from_errno ();
 	}
 	
 	if (gnome_mime_dir.system_dir){
 		list = gnome_vfs_get_registered_mime_types ();
-		if (list == NULL) {
-			return;
+		if (list == NULL) {			
+			return GNOME_VFS_ERROR_INTERNAL;
 		}
 		
 		filename = g_concat_dir_and_file (gnome_mime_dir.dirname, "gnome-vfs.mime");
@@ -1286,7 +1373,7 @@ write_registered_mime_data (void)
         	remove (filename);
 		file = fopen (filename, "w");
 		if (file == NULL) {
-			return;
+			return gnome_vfs_result_from_errno ();
 		}
 
 		/* Get data in list */
@@ -1305,6 +1392,8 @@ write_registered_mime_data (void)
 
 		gnome_vfs_mime_registered_mime_type_list_free (list);
 	}
+
+	return GNOME_VFS_OK;
 }
 
 
