@@ -143,7 +143,11 @@ add_to_key (char *mime_type, char *def)
 		if (!*def)
 			return;
 
-		mp = g_new (RegexMimePair, 1);
+		/* This was g_new instead of g_new0, but there seems
+		 * to be a bug in the Solaris? version of regcomp that
+		 * requires an initialized regex or it will crash.
+		 */
+		mp = g_new0 (RegexMimePair, 1);
 		if (regcomp (&mp->regex, def, REG_EXTENDED | REG_NOSUB)){
 			g_free (mp);
 			return;
@@ -345,11 +349,9 @@ mime_init (void)
 	mime_extensions [0] = g_hash_table_new (g_str_hash, g_str_equal);
 	mime_extensions [1] = g_hash_table_new (g_str_hash, g_str_equal);
 	
-	/* FIXME bugzilla.eazel.com 796: Looks in gnome-libs prefix instead of gnome-vfs prefix. */
-	gnome_mime_dir.dirname = gnome_unconditional_datadir_file ("mime-info");
+	gnome_mime_dir.dirname = g_strconcat (GNOME_VFS_DATADIR, "/mime-info", NULL);
 	gnome_mime_dir.system_dir = TRUE;
 
-	/* FIXME bugzilla.eazel.com 796: Looks in gnome-libs prefix instead of gnome-vfs prefix. */
 	user_mime_dir.dirname  = gnome_util_home_file ("mime-info");
 	user_mime_dir.system_dir = FALSE;
 
