@@ -41,45 +41,13 @@
 #include <stdio.h>
 #include <ctype.h>
 
+#include <stdlib.h> /* for mkstemp */
+#include <unistd.h> /* for close */
+
 #include "gnome-vfs.h"
 #include "gnome-vfs-private.h"
 
-
-GList *
-gnome_vfs_string_list_from_string_array (const gchar * const array[])
-{
-	GList *list;
-	guint i;
 
-	if (array == NULL)
-		return NULL;
-
-	for (i = 0; array[i] != NULL; i++)
-		;
-
-	list = NULL;
-	do {
-		i--;
-		list = g_list_prepend (list, g_strdup (array[i]));
-	} while (i > 0);
-
-	return list;
-}
-
-void
-gnome_vfs_free_string_list (GList *list)
-{
-	GList *p;
-
-	if (list == NULL)
-		return;
-
-	for (p = list; p != NULL; p = p->next)
-		g_free (list->data);
-	g_list_free (list);
-}
-
-
 /* Canonicalize path, and return a new path.  Do everything in situ.  The new
    path differs from path in:
 
@@ -320,16 +288,21 @@ gnome_vfs_create_temp (const gchar *prefix,
 	GnomeVFSHandle *handle;
 	GnomeVFSResult result;
 	gchar *name;
+	gint fd;
 
 	while (1) {
-		name = tempnam (NULL, prefix);
-		if (name == NULL)
+		name = g_strdup_printf("%sXXXXXX", prefix);
+		fd = mkstemp(name);
+
+		if (fd < 0)
 			return GNOME_VFS_ERROR_INTERNAL;
 
-		result = gnome_vfs_create
+		fchmod(fd, 0600);
+		close(fd);
+
+		result = gnome_vfs_open
 			(&handle, name,
-			 GNOME_VFS_OPEN_WRITE | GNOME_VFS_OPEN_READ,
-			 TRUE, 0600);
+			 GNOME_VFS_OPEN_WRITE | GNOME_VFS_OPEN_READ);
 
 		if (result == GNOME_VFS_OK) {
 			*name_return = name;
