@@ -45,8 +45,8 @@ BEGIN_GNOME_DECLS
 #include "gnome-vfs-list-sort.h"
 #include "gnome-vfs-messages.h"
 #include "gnome-vfs-method.h"
+#include "gnome-vfs-cancellable-ops.h"
 #include "gnome-vfs-parse-ls.h"
-#include "gnome-vfs-private-ops.h"
 #include "gnome-vfs-regexp-filter.h"
 #include "gnome-vfs-seekable.h"
 #include "gnome-vfs-shellpattern-filter.h"
