@@ -41,7 +41,7 @@
  * which is the string representation to be used in printf style expressions.
  * This is without the %, so for example for long it would be "ld"
  */
-#include <gnome-vfs-file-size.h>
+#include <libgnomevfs/gnome-vfs-file-size.h>
 
 
 /* Basic enumerations.  */
@@ -88,6 +88,7 @@ enum GnomeVFSResult {
 	GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM,
 	GNOME_VFS_ERROR_NOT_SAME_FILE_SYSTEM,
 	GNOME_VFS_ERROR_NAME_TOO_LONG,
+	GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE,
 	GNOME_VFS_NUM_ERRORS
 };
 typedef enum GnomeVFSResult GnomeVFSResult;
@@ -206,30 +207,21 @@ enum GnomeVFSURIHideOptions {
 };
 typedef enum GnomeVFSURIHideOptions GnomeVFSURIHideOptions;
 
-
-/* File information, a la stat(2).  */
-/* FIXME bugzilla.eazel.com 1202: Some of GnomeVFSFileInfo could be private.  */
 
-struct GnomeVFSFileMetadata {
-	gchar *key;
-	gpointer *value;
-	guint value_size;
-};
-typedef struct GnomeVFSFileMetadata GnomeVFSFileMetadata;
 
 /* File flags.  */
 enum GnomeVFSFileFlags {
 	GNOME_VFS_FILE_FLAGS_NONE = 0,
+	/* Whether the file is a symlink.  */
+	GNOME_VFS_FILE_FLAGS_SYMLINK = 1 << 0,
 	/* Whether the file is on a local file system.  */
 	GNOME_VFS_FILE_FLAGS_LOCAL = 1 << 1,
 };
 typedef enum GnomeVFSFileFlags GnomeVFSFileFlags;
 
 /* Flags indicating what fields in a GnomeVFSFileInfo struct are valid. 
-
    Name is always assumed valid (how else would you have gotten a
-   FileInfo struct otherwise?) and metadata_list is ignored, since
-   it shouldn't be accessed directly anyway.
+   FileInfo struct otherwise?)
  */
 
 enum GnomeVFSFileInfoFields {
@@ -299,13 +291,6 @@ struct GnomeVFSFileInfo {
 	/* MIME type.  */
 	gchar *mime_type;
 
-	/* List of GnomeVFSFileMetadata elements, specifying the metadata for
-           this file.  You should use the `gnome_vfs_file_info_get_metadata()'
-           function to access the values.  Moreover, this does not contain all
-           the metadata for the file, but rather only the data that has been
-           requested in the call that returned this information.  */
-	GList *metadata_list;
-
 	guint refcount;
 };
 typedef struct GnomeVFSFileInfo GnomeVFSFileInfo;
@@ -382,7 +367,6 @@ enum GnomeVFSDirectoryFilterNeeds {
 	GNOME_VFS_DIRECTORY_FILTER_NEEDS_TYPE = 1 << 1,
 	GNOME_VFS_DIRECTORY_FILTER_NEEDS_STAT = 1 << 2,
 	GNOME_VFS_DIRECTORY_FILTER_NEEDS_MIMETYPE = 1 << 3,
-	GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA = 1 << 4
 };
 typedef enum GnomeVFSDirectoryFilterNeeds GnomeVFSDirectoryFilterNeeds;
 
@@ -422,8 +406,8 @@ enum GnomeVFSXferOptions {
 	GNOME_VFS_XFER_EMPTY_DIRECTORIES = 1 << 6,
 	GNOME_VFS_XFER_NEW_UNIQUE_DIRECTORY = 1 << 7,
 	GNOME_VFS_XFER_REMOVESOURCE = 1 << 8,
-	GNOME_VFS_XFER_USE_UNIQUE_NAMES = 1 << 9
-	
+	GNOME_VFS_XFER_USE_UNIQUE_NAMES = 1 << 9,
+	GNOME_VFS_XFER_LINK_ITEMS = 1 << 10
 };
 typedef enum GnomeVFSXferOptions GnomeVFSXferOptions;
 
@@ -518,6 +502,8 @@ enum GnomeVFSXferPhase {
 	GNOME_VFS_XFER_PHASE_SETATTRIBUTES,
 	/* Go to the next file (*) */
 	GNOME_VFS_XFER_PHASE_FILECOMPLETED,
+	/* cleaning up after a move (removing source files, etc.) */
+	GNOME_VFS_XFER_PHASE_CLEANUP,
 	/* Operation finished (*) */
 	GNOME_VFS_XFER_PHASE_COMPLETED,
 	GNOME_VFS_XFER_NUM_PHASES
@@ -597,11 +583,12 @@ typedef gint (* GnomeVFSXferProgressCallback) 	(GnomeVFSXferProgressInfo *info,
 
 typedef struct GnomeVFSAsyncHandle GnomeVFSAsyncHandle;
 
-typedef void	(* GnomeVFSAsyncOpenCallback)	(GnomeVFSAsyncHandle *handle,
+typedef void	(* GnomeVFSAsyncCallback)	(GnomeVFSAsyncHandle *handle,
 						 GnomeVFSResult result,
 						 gpointer callback_data);
 
-typedef GnomeVFSAsyncOpenCallback GnomeVFSAsyncCreateCallback;
+typedef GnomeVFSAsyncCallback GnomeVFSAsyncOpenCallback;
+typedef GnomeVFSAsyncCallback GnomeVFSAsyncCreateCallback;
 
 typedef void	(* GnomeVFSAsyncOpenAsChannelCallback)
 						(GnomeVFSAsyncHandle *handle,
@@ -611,9 +598,7 @@ typedef void	(* GnomeVFSAsyncOpenAsChannelCallback)
 
 typedef GnomeVFSAsyncOpenAsChannelCallback GnomeVFSAsyncCreateAsChannelCallback;
 
-typedef void	(* GnomeVFSAsyncCloseCallback)	(GnomeVFSAsyncHandle *handle,
-						 GnomeVFSResult result,
-						 gpointer callback_data);
+#define GnomeVFSAsyncCloseCallback	GnomeVFSAsyncCallback
 
 typedef void	(* GnomeVFSAsyncReadCallback)	(GnomeVFSAsyncHandle *handle,
 						 GnomeVFSResult result,
@@ -640,6 +625,12 @@ typedef void    (* GnomeVFSAsyncGetFileInfoCallback)
 						 GList *results, /* GnomeVFSGetFileInfoResult* items */
 						 gpointer callback_data);
 
+typedef void	(* GnomeVFSAsyncSetFileInfoCallback)	
+						(GnomeVFSAsyncHandle *handle,
+						 GnomeVFSResult result,
+						 GnomeVFSFileInfo *file_info,
+						 gpointer callback_data);
+
 typedef void	(* GnomeVFSAsyncDirectoryLoadCallback)
 						(GnomeVFSAsyncHandle *handle,
 						 GnomeVFSResult result,
