@@ -466,7 +466,6 @@ struct _DirectoryHandle
 	GnomeVFSURI *uri;
 	DIR *dir;
 	GnomeVFSFileInfoOptions options;
-	const GList *meta_keys;
 
 	struct dirent *current_entry;
 
@@ -481,7 +480,6 @@ static DirectoryHandle *
 directory_handle_new (GnomeVFSURI *uri,
 		      DIR *dir,
 		      GnomeVFSFileInfoOptions options,
-		      const GList *meta_keys,
 		      const GnomeVFSDirectoryFilter *filter)
 {
 	DirectoryHandle *result;
@@ -511,7 +509,6 @@ directory_handle_new (GnomeVFSURI *uri,
 	g_free (full_name);
 
 	result->options = options;
-	result->meta_keys = meta_keys;
 	result->filter = filter;
 
 	return result;
@@ -584,25 +581,40 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 	       struct stat *statptr)
 {
 	struct stat statbuf;
+	gboolean followed_symlink;
 
 	if (statptr == NULL) {
 		statptr = &statbuf;
 	}
 
-	if (options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) {
-		if (stat (full_name, statptr) != 0) {
-			/* failed to resolve the link or some other problem */
-			return gnome_vfs_result_from_errno ();
-		}
-	} else if (lstat (full_name, statptr) != 0) {
+	if (lstat (full_name, statptr) != 0) {
 		return gnome_vfs_result_from_errno ();
 	}
 
-	if (S_ISLNK (statptr->st_mode)) {
-		/* we are dealing with a symlink and the follow flag is off */
+	if ((options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) && S_ISLNK (statptr->st_mode)) {
+		if (stat (full_name, statptr) != 0) {
+			if (errno == ENOENT) {
+				/* its a broken symlink, revert to the lstat */
+				lstat (full_name, statptr);
+				followed_symlink = TRUE;
+			} else {
+				/* go straight to jail, do not pass GO, do not collect $200 */
+				return gnome_vfs_result_from_errno();
+			}
+		} else {
+			followed_symlink = TRUE;
+		}		
+	} else {
+		/* follow links wasn't on, or this wasn't a symlink */
+		followed_symlink = FALSE;
+	}
 
-		g_assert ((options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) == 0);
-		
+	gnome_vfs_stat_to_file_info (file_info, statptr);
+	GNOME_VFS_FILE_INFO_SET_SYMLINK (file_info, followed_symlink);
+
+	GNOME_VFS_FILE_INFO_SET_LOCAL (file_info, TRUE);
+
+	if (S_ISLNK (statptr->st_mode)) {
 		file_info->type = GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK;
 		file_info->symlink_name = read_link (full_name);
 		if (file_info->symlink_name == NULL) {
@@ -611,9 +623,6 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 		file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME;
 	}
 
-	gnome_vfs_stat_to_file_info (file_info, statptr);
-	GNOME_VFS_FILE_INFO_SET_LOCAL (file_info, TRUE);
-
 	return GNOME_VFS_OK;
 }
 
@@ -643,7 +652,6 @@ do_open_directory (GnomeVFSMethod *method,
 		   GnomeVFSMethodHandle **method_handle,
 		   GnomeVFSURI *uri,
 		   GnomeVFSFileInfoOptions options,
-		   const GList *meta_keys,
 		   const GnomeVFSDirectoryFilter *filter,
 		   GnomeVFSContext *context)
 {
@@ -662,7 +670,6 @@ do_open_directory (GnomeVFSMethod *method,
 	*method_handle
 		= (GnomeVFSMethodHandle *) directory_handle_new (uri, dir,
 								 options,
-								 meta_keys,
 								 filter);
 
 	return GNOME_VFS_OK;
@@ -723,8 +730,7 @@ read_directory (DirectoryHandle *handle,
 	    && (filter_needs
 		  & (GNOME_VFS_DIRECTORY_FILTER_NEEDS_TYPE
 		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_STAT
-		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_MIMETYPE
-		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA)) == 0){
+		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_MIMETYPE)) == 0){
 		if (!gnome_vfs_directory_filter_apply (filter, info)) {
 			*skip = TRUE;
 			return GNOME_VFS_OK;
@@ -743,8 +749,7 @@ read_directory (DirectoryHandle *handle,
 	if (filter != NULL
 	    && !filter_called
 	    && (filter_needs
-		  & (GNOME_VFS_DIRECTORY_FILTER_NEEDS_MIMETYPE
-		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA)) == 0) {
+		  & GNOME_VFS_DIRECTORY_FILTER_NEEDS_MIMETYPE) == 0) {
 		if (! gnome_vfs_directory_filter_apply (filter, info)) {
 			*skip = TRUE;
 			return GNOME_VFS_OK;
@@ -757,8 +762,7 @@ read_directory (DirectoryHandle *handle,
 	}
 
 	if (filter != NULL
-	    && !filter_called
-	    && (filter_needs & GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA) == 0) {
+	    && !filter_called) {
 		if (!gnome_vfs_directory_filter_apply (filter, info)) {
 			*skip = TRUE;
 			return GNOME_VFS_OK;
@@ -766,8 +770,6 @@ read_directory (DirectoryHandle *handle,
 		filter_called = TRUE;
 	}
 
-	gnome_vfs_set_meta_for_list (info, full_name, handle->meta_keys);
-
 	if (filter != NULL && !filter_called) {
 		if (!gnome_vfs_directory_filter_apply (filter, info)) {
 			*skip = TRUE;
@@ -808,7 +810,6 @@ do_get_file_info (GnomeVFSMethod *method,
 		  GnomeVFSURI *uri,
 		  GnomeVFSFileInfo *file_info,
 		  GnomeVFSFileInfoOptions options,
-		  const GList *meta_keys,
 		  GnomeVFSContext *context)
 {
 	GnomeVFSResult result;
@@ -834,8 +835,6 @@ do_get_file_info (GnomeVFSMethod *method,
 		get_mime_type (file_info, full_name, options, &statbuf);
 	}
 
-	gnome_vfs_set_meta_for_list (file_info, full_name, meta_keys);
-
 	g_free (full_name);
 
 	return GNOME_VFS_OK;
@@ -846,7 +845,6 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 			      GnomeVFSMethodHandle *method_handle,
 			      GnomeVFSFileInfo *file_info,
 			      GnomeVFSFileInfoOptions options,
-			      const GList *meta_keys,
 			      GnomeVFSContext *context)
 {
 	FileHandle *file_handle;
@@ -875,8 +873,6 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 	if (options & GNOME_VFS_FILE_INFO_GET_MIME_TYPE)
 		get_mime_type (file_info, full_name, options, &statbuf);
 
-	gnome_vfs_set_meta_for_list (file_info, full_name, meta_keys);
-
 	g_free (full_name);
 
 	return GNOME_VFS_OK;
@@ -971,7 +967,7 @@ do_find_directory (GnomeVFSMethod *method,
 		return GNOME_VFS_ERROR_CANCELLED;
 	}
 
-	retval = stat (full_name_near, &near_item_stat);
+	retval = lstat (full_name_near, &near_item_stat);
 	g_free (full_name_near);
 	if (retval != 0)
 		return gnome_vfs_result_from_errno ();
@@ -979,7 +975,7 @@ do_find_directory (GnomeVFSMethod *method,
 	if (gnome_vfs_context_check_cancellation (context))
 		return GNOME_VFS_ERROR_CANCELLED;
 
-	retval = stat (home_directory, &home_volume_stat);
+	retval = lstat (home_directory, &home_volume_stat);
 	if (retval != 0)
 		return gnome_vfs_result_from_errno ();
 
@@ -1204,7 +1200,7 @@ do_check_same_fs (GnomeVFSMethod *method,
 	gint retval;
 
 	full_name_a = get_path_from_uri (a);
-	retval = stat (full_name_a, &sa);
+	retval = lstat (full_name_a, &sa);
 	g_free (full_name_a);
 
 	if (retval != 0)
@@ -1214,7 +1210,7 @@ do_check_same_fs (GnomeVFSMethod *method,
 		return GNOME_VFS_ERROR_CANCELLED;
 
 	full_name_b = get_path_from_uri (b);
-	retval = stat (full_name_b, &sb);
+	retval = lstat (full_name_b, &sb);
 	g_free (full_name_b);
 
 	if (retval != 0)
