@@ -44,14 +44,16 @@
 
 #include <gnome-xml/parser.h>
 #include <gnome-xml/tree.h>
+#include <gnome-xml/xmlmemory.h>
 
 #include "gnome-vfs.h"
 #include "gnome-vfs-private.h"
+#include "gnome-vfs-mime.h"
 
 #include "http-method.h"
 
 
-#if 1				/* FIXME bugzilla.eazel.com 1138 */
+#if 0				/* FIXME bugzilla.eazel.com 1138 */
 #include <stdio.h>
 #define DEBUG_HTTP(x)				\
 	do {					\
@@ -59,6 +61,9 @@
 		printf x;			\
 		putchar ('\n');			\
 	} while (0);
+#else
+static int nothing;
+#define DEBUG_HTTP(x) nothing = 1;
 #endif
 
 
@@ -259,11 +264,16 @@ http_status_to_vfs_result (guint status)
 
 	/* FIXME bugzilla.eazel.com 1163 */
 	switch (status) {
-	case 401:
-	case 407:
+	case HTTP_STATUS_UNAUTHORIZED:
+	case HTTP_STATUS_PROXY_AUTH_REQUIRED:
 		return GNOME_VFS_ERROR_ACCESS_DENIED;
-	case 404:
+	case HTTP_STATUS_NOT_FOUND:
 		return GNOME_VFS_ERROR_NOT_FOUND;
+	case HTTP_STATUS_METHOD_NOT_ALLOWED:
+	case HTTP_STATUS_BAD_REQUEST:
+	case HTTP_STATUS_NOT_IMPLEMENTED:
+	case HTTP_STATUS_UNSUPPORTED_VERSION:
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	default:
 		return GNOME_VFS_ERROR_GENERIC;
 	}
@@ -543,6 +553,54 @@ create_handle (HttpFileHandle **handle_return,
 	return result;
 }
 
+/* BASE64 code ported from neon (http://www.webdav.org/neon) */
+static const gchar b64_alphabet[65] = {
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+    "abcdefghijklmnopqrstuvwxyz"
+    "0123456789+/=" };
+
+static gchar *base64( const gchar *text ) {
+    /* The tricky thing about this is doing the padding at the end,
+     *      * doing the bit manipulation requires a bit of concentration only */
+    gchar *buffer, *point;
+    gint inlen, outlen;
+
+    /* Use 'buffer' to store the output. Work out how big it should be...
+     *      * This must be a multiple of 4 bytes */
+
+    inlen = strlen( text );
+    outlen = (inlen*4)/3;
+    if( (inlen % 3) > 0 ) /* got to pad */
+        outlen += 4 - (inlen % 3);
+
+    buffer = g_malloc( outlen + 1 ); /* +1 for the \0 */
+
+    /* now do the main stage of conversion, 3 bytes at a time,
+     *      * leave the trailing bytes (if there are any) for later */
+
+    for( point=buffer; inlen>=3; inlen-=3, text+=3 ) {
+        *(point++) = b64_alphabet[ (*text)>>2 ];
+        *(point++) = b64_alphabet[ ((*text)<<4 & 0x30) | (*(text+1))>>4 ];
+        *(point++) = b64_alphabet[ ((*(text+1))<<2 & 0x3c) | (*(text+2))>>6 ];
+        *(point++) = b64_alphabet[ (*(text+2)) & 0x3f ];
+    }
+
+    /* Now deal with the trailing bytes */
+    if( inlen ) {
+        /* We always have one trailing byte */
+        *(point++) = b64_alphabet[ (*text)>>2 ];
+        *(point++) = b64_alphabet[ ( ((*text)<<4 & 0x30) |
+                                     (inlen==2?(*(text+1))>>4:0) ) ];
+        *(point++) = (inlen==1?'=':b64_alphabet[ (*(text+1))<<2 & 0x3c ] );
+        *(point++) = '=';
+    }
+
+    /* Null-terminate */
+    *point = '\0';
+
+    return buffer;
+}
+
 static GnomeVFSResult
 make_request (HttpFileHandle **handle_return,
 	      GnomeVFSURI *uri,
@@ -594,8 +652,22 @@ make_request (HttpFileHandle **handle_return,
 	g_string_append (request, " HTTP/1.0\r\n");
 
 	/* `Host:' header.  */
-	g_string_sprintfa (request, "Host: %s:%d\r\n",
+	if(toplevel_uri->host_port && toplevel_uri->host_port != 80)
+		g_string_sprintfa (request, "Host: %s:%d\r\n",
 			   toplevel_uri->host_name, toplevel_uri->host_port);
+	else
+		g_string_sprintfa (request, "Host: %s\r\n",
+			   toplevel_uri->host_name);
+
+	/* Basic authentication */
+	if(toplevel_uri->user_name) {
+		gchar *raw = g_strdup_printf("%s:%s", toplevel_uri->user_name,
+				toplevel_uri->password?toplevel_uri->password:"");
+		gchar *enc = base64(raw);
+		g_string_sprintfa(request, "Authorization: Basic %s\n", enc);
+		g_free(enc);
+		g_free(raw);
+	}
 
 	/* `Accept:' header.  */
 	g_string_append (request, "Accept: */*\r\n");
@@ -685,11 +757,11 @@ do_open (GnomeVFSMethod *method,
 	GnomeVFSResult result = GNOME_VFS_OK;
 
 	g_return_val_if_fail (uri->parent == NULL, GNOME_VFS_ERROR_INVALID_URI);
-	g_return_val_if_fail (mode == GNOME_VFS_OPEN_READ || 
-						mode == GNOME_VFS_OPEN_WRITE,
+	g_return_val_if_fail (!(mode & GNOME_VFS_OPEN_READ && 
+						mode & GNOME_VFS_OPEN_WRITE),
 			      GNOME_VFS_ERROR_INVALID_OPEN_MODE);
 
-	if(mode == GNOME_VFS_OPEN_READ) {
+	if(mode & GNOME_VFS_OPEN_READ) {
 		result = make_request (&handle, uri, "GET", NULL, NULL,
 			       	context);
 	} else {
@@ -711,7 +783,7 @@ do_create (GnomeVFSMethod *method,
      GnomeVFSContext *context)
 {
 	/* FIXME bugzilla.eazel.com 1159: do we need to do something more intelligent here? */
-	return do_open(method, method_handle, uri, mode, context);
+	return do_open(method, method_handle, uri, GNOME_VFS_OPEN_WRITE, context);
 }
 
 static GnomeVFSResult
@@ -732,12 +804,13 @@ do_close (GnomeVFSMethod *method,
 	if (handle->to_be_written != NULL) {
 		GnomeVFSURI *uri = handle->uri;
 		GByteArray *bytes = handle->to_be_written;
+		GnomeVFSFileSize bytes_written;
 
 		result = make_request(&handle, uri, "PUT", bytes, NULL, context);
 
 		if (result == GNOME_VFS_OK) {
 			result = gnome_vfs_iobuf_write (handle->iobuf, bytes->data,
-					bytes->len, NULL /* we don't care how big it was */);
+					bytes->len, &bytes_written);
 		}
 	}
 
@@ -860,6 +933,7 @@ process_propfind_propstat(xmlNodePtr node, GnomeVFSFileInfo *file_info)
 #endif
 
 				}
+				xmlFree(nc);
 			}
 			if(!strcmp((char *)l->name, "resourcetype")) {
 				file_info->valid_fields |= 
@@ -873,6 +947,7 @@ process_propfind_propstat(xmlNodePtr node, GnomeVFSFileInfo *file_info)
 						GNOME_VFS_FILE_TYPE_DIRECTORY;
 					g_free(file_info->mime_type);
 					file_info->mime_type = g_strdup("x-special/webdav-directory");
+					file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
 				}
 			}
 			/* FIXME: 
@@ -887,20 +962,44 @@ process_propfind_propstat(xmlNodePtr node, GnomeVFSFileInfo *file_info)
 }
 
 static GnomeVFSFileInfo *
-process_propfind_response(xmlNodePtr n, gchar *uri_string)
+process_propfind_response(xmlNodePtr n, GnomeVFSURI *base_uri)
 {
 	GnomeVFSFileInfo *file_info = gnome_vfs_file_info_new();
+	GnomeVFSURI *second_base = gnome_vfs_uri_append_path (base_uri, "/");
 
 	file_info->valid_fields = GNOME_VFS_FILE_INFO_FIELDS_NONE;
 
 	gnome_vfs_file_info_init(file_info);
 	while(n != NULL) {
 		if(!strcmp((char *)n->name, "href")) {
-			gchar *nodecontent = gnome_vfs_unescape_string(xmlNodeGetContent(n), "/");
-			gint len;
+			gchar *nc = xmlNodeGetContent(n);
+			gchar *nodecontent = gnome_vfs_unescape_string(nc, "/");
+			xmlFree(nc);
+			if(nodecontent && *nodecontent) {
+				gint len;
+				GnomeVFSURI *uri = gnome_vfs_uri_new(nodecontent);
+
+				if (gnome_vfs_uri_equal (base_uri, uri) ||
+				    gnome_vfs_uri_equal (second_base, uri) ||
+				    !strcmp (base_uri->text, uri->text) ||
+				    !strcmp (second_base->text, uri->text)) {
+				file_info->name = NULL;  /* this file is the . directory */
+
+				} else {
+					file_info->name = gnome_vfs_uri_extract_short_name(uri);
+					gnome_vfs_uri_unref(uri);
+
+					len = strlen(file_info->name)-1;
+					if(file_info->name[len] == '/') {
+						/* trim trailing `/` - it confuses stuff */
+						file_info->name[len] = '\0';
+					}
+				}
+			} else {
+				g_warning("got href without contents in PROPFIND response");
+			}
+
 #if 0
-			g_print("  found href=\"%s\"\n", nodecontent);
-#endif
 			if(!strncmp(uri_string, nodecontent, strlen(uri_string))) {
 				/* our DAV server is prepending the 
 				 * current path
@@ -914,11 +1013,7 @@ process_propfind_response(xmlNodePtr n, gchar *uri_string)
 				file_info->name = 
 					g_strdup(nodecontent);
 			}
-			len = strlen(file_info->name)-1;
-			if(file_info->name[len] == '/') {
-				/* trim trailing `/` - it confuses stuff */
-				file_info->name[len] = '\0';
-			}
+#endif
 			g_free(nodecontent);
 		} else if(!strcmp((char *)n->name, "propstat")) {
 			//g_print("  got <propstat>\n");
@@ -935,6 +1030,9 @@ process_propfind_response(xmlNodePtr n, gchar *uri_string)
 		}
 		n = n->next;
 	}
+
+	gnome_vfs_uri_unref (second_base);
+
 	return file_info;
 }
 
@@ -953,30 +1051,15 @@ make_propfind_request (HttpFileHandle **handle_return,
 	xmlParserCtxtPtr parserContext;
 	xmlDocPtr doc = NULL;
 	xmlNodePtr cur = NULL;
-	gchar *raw_uri = gnome_vfs_uri_to_string (uri,
-                GNOME_VFS_URI_HIDE_USER_NAME
-                |GNOME_VFS_URI_HIDE_PASSWORD
-                |GNOME_VFS_URI_HIDE_HOST_NAME
-                |GNOME_VFS_URI_HIDE_HOST_PORT
-                |GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
-	gchar *unescaped_uri, *uri_string;
 	gchar *extraheaders = g_strdup_printf("Depth: %d\r\n", depth);
+	gchar *raw_uri;
+	gchar *unescaped_uri_string;
+	GnomeVFSURI *unescaped_uri;
 
 	GByteArray *request = g_byte_array_new();
 	gchar *request_str = "<?xml version=\"1.0\" encoding=\"utf-8\" ?>"
 		"<D:propfind xmlns:D=\"DAV:\"><D:allprop/></D:propfind>";
 
-	unescaped_uri = gnome_vfs_unescape_string(raw_uri, "/");
-	g_free(raw_uri);
-
-	if(unescaped_uri[strlen(unescaped_uri)-1] == '/') {
-		uri_string = unescaped_uri;
-	} else {
-		uri_string = g_strconcat(unescaped_uri, "/", NULL);
-		g_free(unescaped_uri);
-	}
-
-
 	request = g_byte_array_append(request, request_str, 
 			strlen(request_str));
 
@@ -986,7 +1069,7 @@ make_propfind_request (HttpFileHandle **handle_return,
 			extraheaders, context);
 
 	if(result == GNOME_VFS_OK && handle->server_status != 207) { /* Multi-Status */
-		g_warning(_("HTTP server returned an invalid PROPFIND response"));
+		g_message(_("HTTP server returned an invalid PROPFIND response"));
 		result = GNOME_VFS_ERROR_NOT_SUPPORTED;
 	}
 
@@ -1028,11 +1111,17 @@ make_propfind_request (HttpFileHandle **handle_return,
 
 	cur = cur->childs;
 
+	raw_uri = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+	unescaped_uri_string = gnome_vfs_unescape_string(raw_uri, "/");
+	unescaped_uri = gnome_vfs_uri_new(unescaped_uri_string);
+	g_free(raw_uri);
+	g_free(unescaped_uri_string);
+
 	while(cur != NULL) {
 		if(!strcmp((char *)cur->name, "response")) {
 			GnomeVFSFileInfo *file_info =
 				process_propfind_response(cur->childs, 
-					uri_string);
+					unescaped_uri);
 			/* if the file has a filename or we're doing a PROPFIND on a single 
 			 * resource... */
 			if(file_info->name || depth==0) { 
@@ -1046,8 +1135,9 @@ make_propfind_request (HttpFileHandle **handle_return,
 		cur = cur->next;
 	}
 
+	gnome_vfs_uri_unref (unescaped_uri);
+
 	g_free(buffer);
-	g_free(uri_string);
 	g_free(extraheaders);
 
 	xmlFreeParserCtxt(parserContext);
@@ -1060,13 +1150,24 @@ do_open_directory(GnomeVFSMethod *method,
 	GnomeVFSMethodHandle **method_handle,
 	GnomeVFSURI *uri,
 	GnomeVFSFileInfoOptions options,
-	const GList *meta_keys,
 	const GnomeVFSDirectoryFilter *filter,
 	GnomeVFSContext *context) 
 {
 	/* TODO move to using the gnome_vfs_file_info_list family of functions */
+	GnomeVFSResult result;
 
-	return make_propfind_request((HttpFileHandle **)method_handle, uri, 1, context);
+	result = make_propfind_request((HttpFileHandle **)method_handle, uri, 1, context);
+	if (result == GNOME_VFS_ERROR_NOT_FOUND) { /* 404 not found */
+		if(uri->text && *uri->text &&
+				uri->text[strlen(uri->text)-1] != '/') {
+			GnomeVFSURI *tmpuri = gnome_vfs_uri_append_path(uri, "/");
+			result = do_open_directory(method, method_handle, tmpuri, options, filter, context);
+			gnome_vfs_uri_unref(tmpuri);
+
+		}
+	}
+
+	return result;
 }
 
 static GnomeVFSResult
@@ -1109,6 +1210,12 @@ do_read_directory (GnomeVFSMethod *method,
 			file_info->size = original_info->size;
 			*/
 			memcpy(file_info, original_info, sizeof(*file_info));
+			if(!file_info->mime_type) {
+				/* we didn't get a mime type - lets guess */
+				file_info->mime_type = g_strdup(gnome_vfs_mime_type_from_name_or_default (file_info->name, "text/plain"));
+				file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
+
+			}
 			found_entry = TRUE;
 		}
 
@@ -1144,7 +1251,6 @@ get_file_info_from_http_handle (HttpFileHandle *handle,
 	file_info->atime = handle->access_time;
 	file_info->mtime = handle->last_modified;
 	file_info->mime_type = g_strdup (handle->mime_type);
-	file_info->metadata_list = NULL;
 
 	if (handle->size_is_known) {
 		file_info->size = handle->size;
@@ -1173,7 +1279,6 @@ do_get_file_info (GnomeVFSMethod *method,
 		  GnomeVFSURI *uri,
 		  GnomeVFSFileInfo *file_info,
 		  GnomeVFSFileInfoOptions options,
-		  const GList *meta_keys,
 		  GnomeVFSContext *context)
 {
 	HttpFileHandle *handle;
@@ -1181,10 +1286,20 @@ do_get_file_info (GnomeVFSMethod *method,
 
 	result = make_request (&handle, uri, "HEAD", NULL, NULL, 
 		       	context);
-	if (result != GNOME_VFS_OK)
-		return result;
+	if (result != GNOME_VFS_OK) {
+		if (result == GNOME_VFS_ERROR_NOT_FOUND) { /* 404 not found */
+			if(uri->text && *uri->text &&
+					uri->text[strlen(uri->text)-1] != '/') {
+				GnomeVFSURI *tmpuri = gnome_vfs_uri_append_path(uri, "/");
+				result = do_get_file_info(method, tmpuri, file_info, options, context);
+				gnome_vfs_uri_unref(tmpuri);
 
-	presult = get_file_info_from_http_handle (handle, file_info, options);
+			}
+		}
+		return result;
+	}
+
+	get_file_info_from_http_handle (handle, file_info, options);
 
 	presult = make_propfind_request(&handle, uri, 0, context);
 	if (presult == GNOME_VFS_OK) {
@@ -1193,30 +1308,37 @@ do_get_file_info (GnomeVFSMethod *method,
 
 		if(webdav_file_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_TYPE) {
 			file_info->type = webdav_file_info->type;
-			file_info->valid_fields &= GNOME_VFS_FILE_INFO_FIELDS_TYPE;
+			file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_TYPE;
 		}	
 
 		if(webdav_file_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE) {
 			g_free(file_info->mime_type);
 			file_info->mime_type = g_strdup(webdav_file_info->mime_type);
-			file_info->valid_fields &= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
+			file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
 		}	
 
 		if(webdav_file_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE) {
 			file_info->size = webdav_file_info->size;
-			file_info->valid_fields &= GNOME_VFS_FILE_INFO_FIELDS_SIZE;
+			file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SIZE;
 		}	
 
+		file_info->valid_fields &= ~GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS;
+		file_info->valid_fields &= ~GNOME_VFS_FILE_INFO_FIELDS_FLAGS;
+
 		#if 0
 		memcpy(file_info, handle->files->data, sizeof(*file_info));
 		#endif
 		g_free(handle->files->data);
 		g_list_free(handle->files);
 		handle->files = NULL;
+		/*
 		file_info->name = g_strdup (g_basename (handle->uri_string));
-		if (file_info->name == NULL)
+		if (file_info->name == NULL) {
 			file_info->name = g_strdup ("");
+		}
+		*/
 	}
+	file_info->name = gnome_vfs_uri_extract_short_name(uri);
 
 	http_handle_close (handle, context);
 
@@ -1228,13 +1350,12 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 			      GnomeVFSMethodHandle *method_handle,
 			      GnomeVFSFileInfo *file_info,
 			      GnomeVFSFileInfoOptions options,
-			      const GList *meta_keys,
 			      GnomeVFSContext *context)
 {
 	HttpFileHandle *handle;
 
 	return do_get_file_info(method, ((HttpFileHandle *)method_handle)->uri, 
-			file_info, options, meta_keys, context);
+			file_info, options, context);
 
 	handle = (HttpFileHandle *) method_handle;
 	return get_file_info_from_http_handle (handle, file_info, options);
@@ -1269,7 +1390,48 @@ static GnomeVFSResult do_remove_directory(GnomeVFSMethod * method,
 	return result;
 }
 
-/* do_move goes here */
+/* a strcmp that doesn't barf on NULLs */
+static gint
+my_strcmp(const gchar *a, const gchar *b) {
+	if( (!a && b) || (a && !b) ) return 1;
+	if( (!a && !b) ) return 0;
+	return strcmp(a,b);
+}
+
+static GnomeVFSResult
+do_move (GnomeVFSMethod *method,
+	 GnomeVFSURI *old_uri,
+	 GnomeVFSURI *new_uri,
+	 gboolean force_replace,
+	 GnomeVFSContext *context)
+{
+	/* MOVE /path1 HTTP/1.0
+	 * Destination: /path2 */
+	HttpFileHandle *handle;
+	GnomeVFSResult result;
+
+	gchar *destpath, *destheader;
+
+	if(my_strcmp(gnome_vfs_uri_get_scheme(old_uri), gnome_vfs_uri_get_scheme(new_uri)) ||
+		my_strcmp(gnome_vfs_uri_get_host_name(old_uri), gnome_vfs_uri_get_host_name(new_uri)) ||
+	  	my_strcmp(gnome_vfs_uri_get_user_name(old_uri), gnome_vfs_uri_get_user_name(new_uri)) ||
+	  	my_strcmp(gnome_vfs_uri_get_password(old_uri), gnome_vfs_uri_get_password(new_uri)) ||
+		(gnome_vfs_uri_get_host_port(old_uri) != gnome_vfs_uri_get_host_port(new_uri))) {
+
+		/* the host/username/password/port are different */
+		return GNOME_VFS_ERROR_NOT_SAME_FILE_SYSTEM;
+	}
+
+	destpath = gnome_vfs_uri_to_string(new_uri, GNOME_VFS_URI_HIDE_USER_NAME|GNOME_VFS_URI_HIDE_PASSWORD);
+	destheader = g_strdup_printf("Destination: %s\r\n", destpath);
+
+
+	result = make_request (&handle, old_uri, "MOVE", NULL, destheader, context);
+	http_handle_close (handle, context);
+	return result;
+	//return GNOME_VFS_ERROR_NOT_SUPPORTED;
+}
+
 
 static GnomeVFSResult do_unlink(GnomeVFSMethod * method,
              GnomeVFSURI * uri, GnomeVFSContext * context) {
@@ -1294,7 +1456,7 @@ static GnomeVFSMethod method = {
 	do_is_local,
 	do_make_directory,
 	do_remove_directory,
-	NULL /* do_move */,
+	do_move,
 	do_unlink,
 	NULL,
 	NULL, /* truncate */
