@@ -112,7 +112,7 @@ print_list (GnomeVFSDirectoryList *list)
 
 		printf ("  File `%s'%s (%s, %s), size %ld, mode %04o\n",
 			info->name,
-			(info->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK) ? " [link]" : "",
+			GNOME_VFS_FILE_INFO_SYMLINK (info) ? " [link]" : "",
 			type_to_string (info->type),
 			mime_type,
 			(glong) info->size,
@@ -226,7 +226,6 @@ main (int argc, char **argv)
 		 (GNOME_VFS_FILE_INFO_GET_MIME_TYPE
 		  | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE
 		  | GNOME_VFS_FILE_INFO_FOLLOW_LINKS),
-		 NULL,
 		 NULL);
 
 	if (result == GNOME_VFS_OK && measure_speed) {
